@@ -324,6 +324,7 @@ class TransparentClassroomClient:
                 ('student_emergency_contacts_string_tc', datum.get('emergency_contacts_string')),
                 ('student_program_tc', datum.get('program')),
                 ('student_grade_tc', datum.get('grade')),
+                ('student_first_day_tc', wf_core_data.utils.to_date(datum.get('first_day'))),
                 ('student_last_day_tc', wf_core_data.utils.to_date(datum.get('last_day'))),
                 ('student_exit_reason_tc', datum.get('exit_reason')),
                 ('student_id_alt_tc', datum.get('student_id')),
@@ -1012,6 +1013,171 @@ class TransparentClassroomClient:
             raise ValueError('Data format \'{}\' not recognized'.format(format))
         return school_data
 
+    def fetch_network_form_template_data(
+        self,
+        pull_datetime=None,
+        format='dataframe'
+    ):
+        form_template_data = self.fetch_form_template_data_school(
+            school_id=None,
+            pull_datetime=None,
+            format=format
+        )
+        if format == 'dataframe':
+            form_template_data = form_template_data.droplevel('school_id_tc')
+        elif format == 'list':
+            for element in form_template_data:
+                element.pop('school_id_tc')
+        else:
+            raise ValueError('Data format \'{}\' not recognized'.format(format))
+        return form_template_data
+
+    def fetch_form_template_data(
+        self,
+        school_ids=None,
+        pull_datetime=None,
+        format='dataframe'
+    ):
+        pull_datetime = wf_core_data.utils.to_datetime(pull_datetime)
+        if pull_datetime is None:
+            pull_datetime = datetime.datetime.now(tz=datetime.timezone.utc)
+        if school_ids is None:
+            school_ids=self.fetch_school_ids()
+        logger.info('Fetching form template data from Transparent Classroom for {} schools'.format(len(school_ids)))
+        form_template_data = list()
+        for school_id in school_ids:
+            form_template_data_school = self.fetch_form_template_data_school(
+                school_id=school_id,
+                pull_datetime=pull_datetime,
+                format='list'
+            )
+            form_template_data.extend(form_template_data_school)
+        if format == 'dataframe':
+            form_template_data = convert_form_template_data_to_df(form_template_data)
+        elif format == 'list':
+            pass
+        else:
+            raise ValueError('Data format \'{}\' not recognized'.format(format))
+        return form_template_data
+
+    def fetch_form_template_data_school(
+        self,
+        school_id,
+        pull_datetime=None,
+        format='dataframe'
+    ):
+        if school_id is not None:
+            school_id = int(school_id)
+            logger.info('Fetching form template data from Transparent Classroom for school ID {}'.format(school_id))
+        else:
+            logger.info('Fetching form template data from Transparent Classroom for network')
+        pull_datetime = wf_core_data.utils.to_datetime(pull_datetime)
+        if pull_datetime is None:
+            pull_datetime = datetime.datetime.now(tz=datetime.timezone.utc)
+        json_output = self.transparent_classroom_request(
+            'form_templates.json',
+            school_id=school_id
+        )
+        if not isinstance(json_output, list):
+            raise ValueError('Received unexpected response from Transparent Classroom: {}'.format(
+                json_output
+            ))
+        form_template_data = list()
+        for datum in json_output:
+            form_template_datum = OrderedDict([
+                ('school_id_tc', school_id),
+                ('form_template_id_tc', datum.get('id')),
+                ('pull_datetime', pull_datetime),
+                ('form_template_name', datum.get('name')),
+                ('widgets', datum.get('widgets'))
+            ])
+            form_template_data.append(form_template_datum)
+        if format == 'dataframe':
+            form_template_data = convert_form_template_data_to_df(form_template_data)
+        elif format == 'list':
+            pass
+        else:
+            raise ValueError('Data format \'{}\' not recognized'.format(format))
+        return form_template_data
+
+    def fetch_form_data(
+        self,
+        school_ids=None,
+        pull_datetime=None,
+        form_template_id=None,
+        format='dataframe'
+    ):
+        pull_datetime = wf_core_data.utils.to_datetime(pull_datetime)
+        if pull_datetime is None:
+            pull_datetime = datetime.datetime.now(tz=datetime.timezone.utc)
+        if school_ids is None:
+            school_ids=self.fetch_school_ids()
+        logger.info('Fetching form data from Transparent Classroom for {} schools'.format(len(school_ids)))
+        form_data = list()
+        for school_id in school_ids:
+            form_data_school = self.fetch_form_data_school(
+                school_id=school_id,
+                pull_datetime=pull_datetime,
+                form_template_id=form_template_id,
+                format='list'
+            )
+            form_data.extend(form_data_school)
+        if format == 'dataframe':
+            form_data = convert_form_data_to_df(form_data)
+        elif format == 'list':
+            pass
+        else:
+            raise ValueError('Data format \'{}\' not recognized'.format(format))
+        return form_data
+
+    def fetch_form_data_school(
+        self,
+        school_id,
+        pull_datetime=None,
+        form_template_id=None,
+        format='dataframe'
+    ):
+        school_id = int(school_id)
+        logger.info('Fetching form template data from Transparent Classroom for school ID {}'.format(school_id))
+        pull_datetime = wf_core_data.utils.to_datetime(pull_datetime)
+        if pull_datetime is None:
+            pull_datetime = datetime.datetime.now(tz=datetime.timezone.utc)
+        json_output = self.transparent_classroom_request(
+            'forms.json',
+            school_id=school_id,
+            params={
+                'form_template_id': form_template_id
+            }
+        )
+        if not isinstance(json_output, list):
+            raise ValueError('Received unexpected response from Transparent Classroom: {}'.format(
+                json_output
+            ))
+        form_data = json_output
+        form_data = list()
+        for datum in json_output:
+            form_datum = OrderedDict([
+                ('school_id_tc', school_id),
+                ('form_id_tc', datum.get('id')),
+                ('student_id_tc', datum.get('child_id')),
+                ('form_template_id_tc', datum.get('form_template_id')),
+                ('pull_datetime', pull_datetime),
+                ('form_state', datum.get('state')),
+                ('form_created', datum.get('created_at')),
+                ('form_updated', datum.get('updated_at')),
+                ('form_last_emailed', datum.get('last_emailed_at')),
+                ('form_due_date', datum.get('due_date')),
+                ('form_fields', datum.get('fields'))
+            ])
+            form_data.append(form_datum)
+        if format == 'dataframe':
+            form_data = convert_form_data_to_df(form_data)
+        elif format == 'list':
+            pass
+        else:
+            raise ValueError('Data format \'{}\' not recognized'.format(format))
+        return form_data
+
     def transparent_classroom_request(
         self,
         endpoint,
@@ -1386,6 +1552,7 @@ def convert_student_data_to_df(student_data):
             'student_approved_adults_string_tc': 'string',
             'student_emergency_contacts_string_tc': 'string',
             'student_notes_tc': 'string',
+            'student_first_day_tc': 'object',
             'student_last_day_tc': 'object',
             'student_exit_reason_tc': 'string',
             'student_exit_survey_id_tc': 'Int64',
@@ -1426,3 +1593,41 @@ def convert_student_parent_data_to_df(student_parent_data):
     })
     student_parent_data_df.set_index(['school_id_tc', 'student_id_tc', 'parent_id_tc'], inplace=True)
     return student_parent_data_df
+
+def convert_form_template_data_to_df(form_template_data):
+    if len(form_template_data) == 0:
+        return pd.DataFrame()
+    form_template_data_df = pd.DataFrame(
+        form_template_data,
+        dtype='object'
+    )
+    form_template_data_df['pull_datetime'] = pd.to_datetime(form_template_data_df['pull_datetime'])
+    form_template_data_df = form_template_data_df.astype({
+            'school_id_tc': 'Int64',
+            'form_template_id_tc': 'int',
+            'form_template_name': 'string'
+    })
+    form_template_data_df.set_index(['school_id_tc', 'form_template_id_tc'], inplace=True)
+    return form_template_data_df
+
+def convert_form_data_to_df(form_data):
+    if len(form_data) == 0:
+        return pd.DataFrame()
+    form_data_df = pd.DataFrame(
+        form_data,
+        dtype='object'
+    )
+    form_data_df['pull_datetime'] = pd.to_datetime(form_data_df['pull_datetime'])
+    form_data_df['form_created'] = pd.to_datetime(form_data_df['form_created'])
+    form_data_df['form_updated'] = pd.to_datetime(form_data_df['form_updated'])
+    form_data_df['form_last_emailed'] = pd.to_datetime(form_data_df['form_last_emailed'])
+    form_data_df['form_due_date'] = pd.to_datetime(form_data_df['form_due_date'])
+    form_data_df = form_data_df.astype({
+            'school_id_tc': 'int',
+            'form_id_tc': 'int',
+            'child_id_tc': 'int',
+            'form_template_id_tc': 'int',
+            'form_state': 'string',
+    })
+    form_data_df.set_index(['school_id_tc', 'form_id_tc'], inplace=True)
+    return form_data_df
